import curses
import curses.textpad
import textwrap


class Focusable():

    def __init__(self):
        self.is_focused = False

    def set_focused(self, focus):
        self.is_focused = focus


class Widget(object):
    _TEXT_WRAP_PADDING = 4

    def __init__(self, window, content, header, rect, config):
        self.window = window
        self.rect = rect
        self.content = content
        # header text
        self.header = header
        # create sub window
        self.subwindow = self.window.subwin(
            self.rect.h, self.rect.w, self.rect.y, self.rect.x)
        self.box = config.getboolean('box', fallback=False)
        self.color = curses.color_pair(config.getint('fg', fallback=11))
        self.header_color = curses.color_pair(
            config.getint('headerbg', fallback=12)) | curses.A_REVERSE | curses.A_BOLD | curses.A_UNDERLINE

    def update_content(self, content):
        self.content = content
        self.display()

    def resize(self, rect):
        ''' Resizes the current widget
        If widget doesn't fit window tries moving it
        '''
        self.subwindow.clear()
        self.subwindow.refresh()

        if rect.w != self.rect.w or rect.h != self.rect.h:
            try:
                self.subwindow.mvwin(rect.y, rect.x)
                self.subwindow.resize(rect.h, rect.w)
            except curses.error:
                # Not enough space for resizing...
                try:
                    self.subwindow.mvderwin(rect.y, rect.x)
                    self.subwindow.resize(rect.h, rect.w)
                except curses.error:
                    pass
                    # raise ValueError("Resizing Failed!")
        else:
            try:
                self.subwindow.mvwin(rect.y, rect.x)
            except curses.error:
                pass
        self.rect = rect
        self.subwindow.refresh()

    def redraw(self):
        ''' Erases all text and re displays '''
        self.subwindow.clear()
        self.display()

    def display(self):
        """ Display the content on window by erasing and adding the text again"""
        self.subwindow.erase()
        if self.content is None or len(self.content) == 0:
            return
        self._draw_items()
        self.subwindow.refresh()

    def draw_header(self):
        has_header = len(self.header) > 0
        if has_header:
            t = textwrap.shorten(self.header, self.rect.w - 2)
            lenght = max(0, (self.rect.w - 2 - len(t)))
            text = t + ' ' * lenght
            self.subwindow.addstr(1, 1, text, self.header_color)
        x = 2 if has_header else 1
        if self.box:
            try:
                self.subwindow.border()
                self.subwindow.refresh()
            except curses.error:
                pass
        return x

    def _draw_items(self):
        ''' Draws content and header '''
        # header
        start_x = self.draw_header()
        # draw items
        for index, item in enumerate(self.content[0:self.rect.h]):
            self.subwindow.addstr(
                index + start_x,
                2,
                textwrap.shorten(str(item), self.rect.w - self._TEXT_WRAP_PADDING),
                self.color)



from .scrollablewidget import ScrollableWidget, Focusable


class SubsWidget(ScrollableWidget, Focusable):

    def __init__(self, window, content, header, rect, config, videos_cache):
        super().__init__(
            window, content, header, rect, config)
        self.cache = videos_cache

    def update_content(self, content, scroll_to_zero=True):
        # show only active content
        c = [channel for channel in content if not self.cache.contains_all(
            [i.link for i in channel.entries])]
        super().update_content(c, scroll_to_zero)

    def set_focused(self, focus):
        if focus:
            self.update_content(self.content, False)
        super().set_focused(focus)



from yet.ui.window import Window
from yet.yt.channel import Subscriptions
from yet.config import config
from yet.config.videoscache import VideosCache


def main():
    try:
        channel_ids = config.get_channel_ids()
    except FileNotFoundError:
        print("Using default channel...")
        channel_ids = ["UCLA_DiR1FfKNvjuUpBHmylQ"]

    try:
        yet_config = config.YetConfig()
    except config.YetConfigException:
        return

    videos_cache = VideosCache(yet_config.get_section("common").getint("clean_cache", fallback=0))
    window = Window(None, yetconfig=yet_config, videoscache=videos_cache)
    Subscriptions(channel_ids, window.update_model)
    window.run()


if __name__ == '__main__':
    main()



"""
YET
"""

from __future__ import (absolute_import, division, print_function)

import os
from sys import version_info


# Version helper
def version_helper():
    if __release__:
        version_string = 'yet {0}'.format(__version__)
    else:
        import subprocess
        version_string = 'yet-master {0}'
        try:
            git_describe = subprocess.Popen(['git', 'describe'],
                                            universal_newlines=True,
                                            cwd=YETDIR,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
            (git_description, _) = git_describe.communicate()
            version_string = version_string.format(git_description.strip('\n'))
        except (OSError, subprocess.CalledProcessError):
            version_string = version_string.format(__version__)
    return version_string


# Information
__license__ = 'GPL3'
__version__ = '0.1.6'
__release__ = False
__author__ = __maintainer__ = 'mushroomrisotto'
__email__ = 'pypi.gib@aleeas.com'

# Constants
YETDIR = os.path.dirname(__file__)
USAGE = '%prog [options] [path]'
VERSION = version_helper()
PY3 = version_info[0] >= 3

# These variables are ignored if the corresponding
# XDG environment variable is non-empty and absolute
# CACHEDIR = os.path.expanduser('~/.cache/yet')
# CONFDIR = os.path.expanduser('~/.config/yet')
# DATADIR = os.path.expanduser('~/.local/share/yet')

# args = None



from yet.config.videoscache import VideosCache
from unittest import TestCase


class TestVideosCache(TestCase):
    TEST_URL = "_TestUrl_"

    def __init__(self):
        super().__init__()
        self.videos_cache = VideosCache()

    def test_videos_cache_add(self):
        self.videos_cache.add_to_cache(self.TEST_URL)
        c = self.videos_cache.contains(self.TEST_URL)
        self.assertTrue(c)

    def test_videos_cache_remove(self):
        self.videos_cache.delete(self.TEST_URL)
        c = self.videos_cache.contains(self.TEST_URL)
        self.assertFalse(c)



from .widget import Widget, Focusable
import curses
import textwrap


class ScrollableWidget(Widget, Focusable):
    ''' Scrollable widget class'''
    UP = -1
    DOWN = 1

    def __init__(self, window, content, header, rect, config):
        super().__init__(
            window, content, header, rect, config)
        Focusable.__init__(self)
        self.top = 0
        self.bottom = 0
        self.max_lines = self.rect.h - 3
        self.current = 0
        self.selectcallback = None
        self.selected_color = curses.color_pair(config.getint('selectionfg', fallback=11)) | curses.A_REVERSE

    def scroll(self, direction):
        """Scrolling the window when pressing up/down arrow keys"""

        if not self.is_focused:
            return

        current_index = self.get_current_index()
        # next cursor position after scrolling
        next_line = self.current + direction

        # Up direction scroll overflow
        # current cursor position is 0, but top position is greater than 0
        if (direction == self.UP) and (self.top > 0 and self.current == 0):
            self.top += direction
        # Down direction scroll overflow
        # next cursor position touch the max lines, but absolute position of max lines could not touch the bottom
        elif (direction == self.DOWN) and (next_line == self.max_lines) and (self.top + self.max_lines < self.bottom):
            self.top += direction
        # Scroll up
        # current cursor position or top position is greater than 0
        elif (direction == self.UP) and (self.top > 0 or self.current > 0):
            self.current = next_line
        # Scroll down
        # next cursor position is above max lines, and absolute position of next cursor could not touch the bottom
        elif (direction == self.DOWN) and (next_line < self.max_lines) and (self.top + next_line < self.bottom):
            self.current = next_line

        # Send index
        if current_index != self.get_current_index():
            if self.selectcallback is not None:
                self.selectcallback(self.get_current_index())
            self.window.move(self.current + 2, self.rect.x)
            self.display()

    def get_current_index(self):
        return self.current + self.top

    def get_current_index_item(self):
        if self.content is None or len(self.content) == 0:
            return None
        return self.content[self.current + self.top]

    def update_content(self, content, scroll_to_zero=True):
        if scroll_to_zero:
            self.current = 0
        super(ScrollableWidget, self).update_content(content)
        self.bottom = len(self.content)

    def set_listener(self, listener):
        self.selectcallback = listener

    def set_focused(self, focused):
        if focused == self.is_focused:
            return
        super().set_focused(focused)
        if self.selectcallback is not None:
            self.selectcallback(self.get_current_index())
        self.window.move(self.current + 2, self.rect.x)
        self.display()

    def resize(self, rect):
        super().resize(rect)
        self.max_lines = self.rect.h - 3

    def _draw_items(self):
        """Display the scrollable content on window"""
        # header
        start_y = self.draw_header()

        # draw items
        for index, item in enumerate(self.content[self.top:self.top + self.max_lines]):
            # Highlight the current cursor line
            color = self.selected_color if index == self.current else self.color
            short_text = textwrap.shorten(
                str(item), self.rect.w - self._TEXT_WRAP_PADDING)
            # until the end of line
            short_text += " " * (self.rect.w - 2 - len(short_text))
            self.subwindow.addstr(
                index + start_y,
                1,
                short_text,
                color)



import os
import json
import configparser
import ast
from pathlib import Path

CONFIG_PATH = os.path.join(Path.home(), ".config", "yet")
SUBS_PATH = os.path.join(CONFIG_PATH, "subscriptions.json")
CONFIG_FILE_PATH = os.path.join(CONFIG_PATH, "yet.conf")
CONFIG_FALLBACK_PATH = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "yet.conf")


def get_videos_dir():
    return os.path.join(Path.home(), "Videos/")


def _get_subs_json():
    """ Reads subscriptions json file """
    try:
        with open(SUBS_PATH) as f:
            return json.load(f)
    except IOError:
        print("%s not found!" % SUBS_PATH)
        return None


def get_channel_ids() -> list:
    """ Returns channel ids from youtube export data """
    json = _get_subs_json()
    if json is None:
        # TODO look for simple text maybe
        raise FileNotFoundError
    channels = [i['snippet']['resourceId']['channelId']
                for i in json if i['snippet']['resourceId']['kind'] == 'youtube#channel']
    return channels


class YetConfigException(Exception):
    pass


class YetConfig(object):
    """
    Configuration for yet
    """

    CONFIG_KEY_COMMON = 'common'
    CONFIG_KEY_YT = 'youtube-dl'
    CONFIG_KEY_KEYBINDINGS = 'keybindings'
    CONFIG_KEY_WIDGET_SUBS = 'subswidget'
    CONFIG_KEY_WIDGET_VIDEO = 'videoswidget'
    CONFIG_KEY_WIDGET_INFO = 'infowidget'
    CONFIG_KEY_WIDGET_BAR = 'infobar'

    def __init__(self):
        self.config = configparser.ConfigParser(allow_no_value=True)
        self.config.read(CONFIG_FILE_PATH)
        if len(self.config.sections()) == 0:
            # check fallback config
            print("Can't find config file {home}. Using default {d}".format(
                home=CONFIG_FILE_PATH, d=CONFIG_FALLBACK_PATH))
            self.config.read(CONFIG_FALLBACK_PATH)
            if len(self.config.sections()) == 0:
                raise YetConfigException("Can't parse or locate config file.")

    def get_section(self, section_key) -> configparser.SectionProxy:
        """
        Returns section proxy, if key is not available raises YetConfigException
        """
        if not self.config.has_section(section_key):
            raise YetConfigException(
                "Config file doesn't contain section: {key}".format(key=section_key))
        return self.config[section_key]

    def get_keybinding(self, key) -> list:
        """
        Returns keybindings for a given key.

        >>> binding = get_keybinding("download")
        [100]
        """
        if not self.config.has_section(self.CONFIG_KEY_KEYBINDINGS):
            raise YetConfigException(
                "Config file doesn't contain section: {key}".format(key=self.CONFIG_KEY_KEYBINDINGS))

        section = self.config[self.CONFIG_KEY_KEYBINDINGS]
        keys = []
        for k in ast.literal_eval(section[key]):
            keys.append(k if type(k) is int else ord(k))
        return keys



import sqlite3 as sq
import datetime
import os
from pathlib import Path


class VideosCache(object):
    CACHE_PATH = os.path.join(Path.home(), ".cache", "yet")
    VIDEOS_CACHE_DB_PATH = os.path.join(CACHE_PATH, "videoscache.db")
    TABLE = "videos_cache"

    def __init__(self, clean_day=0):
        self._create_table_if_not_exists()
        if clean_day > 0:
            self.clear_old_cache(clean_day)

    def _create_table_if_not_exists(self):
        ''' Creates dir and sql table if not exists '''
        if not os.path.exists(self.CACHE_PATH):
            os.makedirs(self.CACHE_PATH)
        with sq.connect(self.VIDEOS_CACHE_DB_PATH) as connection:
            sql = 'create table if not exists ' + self.TABLE + \
                ' (id integer NOT NULL PRIMARY KEY AUTOINCREMENT, link TEXT, date timestamp)'
            connection.execute(sql)

    def add_to_cache(self, url):
        if self.contains(url):
            return
        sql = 'INSERT INTO ' + self.TABLE + ' (link, date) values(?, ?)'
        values = (url, datetime.datetime.today())
        with sq.connect(self.VIDEOS_CACHE_DB_PATH) as connection:
            connection.execute(sql, values)

    def delete(self, url):
        if not self.contains(url):
            return
        sql = 'DELETE FROM ' + self.TABLE + ' WHERE link = ?'
        with sq.connect(self.VIDEOS_CACHE_DB_PATH) as connection:
            connection.execute(sql, (url,))

    def contains(self, url):
        sql = 'SELECT * FROM ' + self.TABLE + ' WHERE link = ?'
        with sq.connect(self.VIDEOS_CACHE_DB_PATH) as connection:
            data = connection.execute(sql, (url,))
            for row in data:
                return True
        return False

    def contains_all(self, urls):
        for url in urls:
            if not self.contains(url):
                return False
        return True

    def clear_cache(self):
        sql = 'DELETE FROM ' + self.TABLE + ';'
        with sq.connect(self.VIDEOS_CACHE_DB_PATH) as connection:
            connection.execute(sql)

    def clear_old_cache(self, days):
        sql = 'DELETE FROM ' + self.TABLE + ' WHERE date < ?'
        date = datetime.datetime.today() - datetime.timedelta(days=days)
        with sq.connect(self.VIDEOS_CACHE_DB_PATH) as connection:
            connection.execute(sql, (date,))



from .infobar import InfoBar
from .rect import Rect
from .scrollablewidget import ScrollableWidget
from .videoswidget import VideosWidget
from .infowidget import InfoWidget
from .widget import Focusable
from .subswidget import SubsWidget
from yet.yt.ytdlwrapper import YtdlWrapper
import curses
import curses.textpad
import subprocess
import webbrowser


class Window(object):

    def __init__(self, model=None, yetconfig=None, videoscache=None):

        self.yet_config = yetconfig
        self.video_index = 0
        self.download = False
        self.subs_widget = None
        self.videos_screen = None
        self.info_screen = None
        self.info_bar = None
        self.widget_placement = [0.2, 0.4, 0.4]
        self.videos_cache = videoscache

        self.window = self.init_curses()
        self.model = model
        # create widgets
        self.widgets = self.init_widgets()

    def init_curses(self):
        window = curses.initscr()
        window.keypad(True)
        curses.noecho()
        # curses.cbreak()
        window.nodelay(False)
        # curses.halfdelay(20)
        curses.start_color()
        curses.use_default_colors()
        for i in range(0, curses.COLORS):
            curses.init_pair(i + 1, i, -1)
        # noinspection PyAttributeOutsideInit
        self.console_height, self.console_width = window.getmaxyx()
        return window

    # noinspection PyAttributeOutsideInit
    def init_widgets(self):
        widgets = []
        # Subs window
        w = round(self.console_width * self.widget_placement[0])
        rect = Rect(w, self.console_height - 1, 0, 0)
        self.subs_widget = SubsWidget(self.window,
                                      None,
                                      'Subscriptions', rect,
                                      self.yet_config.get_section(self.yet_config.CONFIG_KEY_WIDGET_SUBS),
                                      self.videos_cache)
        widgets.append(self.subs_widget)
        self.subs_widget.set_listener(self._on_select_subs)

        # Videos
        w = round(self.console_width * self.widget_placement[1])
        rect2 = Rect(w, self.console_height - 1, rect.w, 0)
        self.videos_widget = VideosWidget(self.window,
                                          [],
                                          'Videos', rect2,
                                          self.yet_config.get_section(self.yet_config.CONFIG_KEY_WIDGET_VIDEO))
        self.videos_widget.set_listener(self._on_select_video)
        widgets.append(self.videos_widget)

        # Info
        w = self.console_width - rect.w - rect2.w
        rect3 = Rect(w, self.console_height - 1, rect2.x + rect2.w, 0)
        self.info_widget = InfoWidget(self.window,
                                      [],
                                      'Info', rect3,
                                      self.yet_config.get_section(self.yet_config.CONFIG_KEY_WIDGET_INFO))
        widgets.append(self.info_widget)

        rect4 = Rect(self.console_width, 1, 0, self.console_height - 1)
        self.info_bar = InfoBar(self.window, None, None, rect4,
                                self.yet_config.get_section(self.yet_config.CONFIG_KEY_WIDGET_BAR))
        widgets.append(self.info_bar)

        # current screen subs
        self.screen_index = 0
        for i in widgets:
            i.display()

        return widgets

    def update_model(self, i, total, model):
        self.model = model
        self.subs_widget.update_content(model)
        self.info_bar.set_info_text(
            "Fetching feed %d/%d" % (i, total), float(i) / total)
        if i == total:
            self.info_bar.set_info_text(
                "Welcome to yet! Select a video to download. Press Q to exit.")
            # set focused
            self.subs_widget.set_focused(True)

    def destroy(self):
        self.window.clear()
        self.window.refresh()
        del self.window
        curses.endwin()

    def run(self):
        """Continue running the TUI until get interrupted"""
        try:
            self._main_loop()
        except KeyboardInterrupt:
            pass
        finally:
            self.destroy()

    def _main_loop(self):
        """Waiting an input
        and run a proper method according to type of input"""
        while True:
            # input
            ch = self.window.getch()
            if ch in self.yet_config.get_keybinding("exit"):  # EXIT
                break
            elif ch in self.yet_config.get_keybinding("up"):
                self._scroll_widget(-1)
            elif ch in self.yet_config.get_keybinding("down"):
                self._scroll_widget(1)
            elif ch in self.yet_config.get_keybinding("left"):
                self._focus_next_widget(-1)
            elif ch in self.yet_config.get_keybinding("right"):
                self._focus_next_widget(1)
            elif ch == curses.KEY_RESIZE:
                self.resize()
            elif ch in self.yet_config.get_keybinding("markasread"):
                self.mark_as_read()
            elif ch in self.yet_config.get_keybinding("selectvideo"):
                self.videos_widget.select_video()
            elif ch in self.yet_config.get_keybinding("open"):
                self.open_in_browser()
            elif ch in self.yet_config.get_keybinding("download"):
                self.yt_download()
            elif ch in self.yet_config.get_keybinding("vlc"):
                self.open_in_vlc()

    # noinspection PyAttributeOutsideInit
    def resize(self):
        """ Resize curses window and all widgets """
        height, width = self.window.getmaxyx()
        if width != self.console_width or height != self.console_height:
            self.window.erase()
            curses.resizeterm(height, width)
            self.window.refresh()
            # set new values
            self.console_width = width
            self.console_height = height

            # subs widget
            rect = Rect(rect=self.subs_widget.rect)
            rect.w = round(self.console_width * self.widget_placement[0])
            rect.h = self.console_height - 1
            self.subs_widget.resize(rect)

            # video
            rect2 = Rect(rect=self.videos_widget.rect)
            rect2.w = round(self.console_width * self.widget_placement[1])
            rect2.h = self.console_height - 1
            rect2.x = rect.w
            self.videos_widget.resize(rect2)

            # info
            rect3 = Rect(rect=self.info_widget.rect)
            rect3.w = self.console_width - rect.w - rect2.w
            rect3.h = self.console_height - 1
            rect3.x = rect2.w + rect2.x
            self.info_widget.resize(rect3)

            # info bar down, h is always 1 , x is always 0
            rect4 = Rect(rect=self.info_bar.rect)
            rect4.w = self.console_width
            rect4.y = self.console_height - 1
            self.info_bar.resize(rect4)

            for i in self.widgets:
                i.display()

    def _scroll_widget(self, direction):
        """ Scroll a scrollable widget with direction """
        w = self.widgets[self.screen_index]
        if isinstance(w, ScrollableWidget):
            w.scroll(direction)

    def _focus_next_widget(self, index):
        """ Focuses next widget """
        widgets = [i for i in self.widgets if isinstance(i, Focusable)]
        w = widgets[self.screen_index]
        if isinstance(w, Focusable):
            w.set_focused(False)
        # set index
        self.screen_index += index
        if self.screen_index < 0:
            self.screen_index = 0
        else:
            self.screen_index %= len(widgets)
        widgets[self.screen_index].set_focused(True)

    def _on_select_subs(self, index):
        """ Called when subscriptions widget selects a channel """

        # channel feed entries for videos
        channel = self.subs_widget.get_current_index_item()
        if channel is None:
            return
        entries = channel.feed.entries
        self.videos_widget.update_content(
            [i for i in entries if not self.videos_cache.contains(i.link)])

    def _on_select_video(self, index):
        """ Called when videos widget selects a video """

        items = self.videos_widget.get_current_index_item().get_extended_info()
        self.info_widget.update_content(items)
        if not self.download:
            self.info_bar.set_info_text(
                "Press D to download -  Space to add to the basket - O to open in browser - V to open in VLC")

    def yt_download(self):
        """ Start downloading videos/s """

        if self.download:
            return
        urls = self.videos_widget.get_urls_to_download()
        if urls is None:
            self.info_bar.set_info_text("Select video/s to download...")
            return
        try:
            ytdl = YtdlWrapper(
                self._yt_hook, self.yet_config.get_section("youtube-dl"))
            ytdl.get(urls)
            self.info_bar.set_info_text("Checking library.")
            if self.yet_config.get_section("common").getboolean("auto_markasread"):
                self.mark_as_read(True)
        except Exception as e:
            self.info_bar.set_info_text(str(e))
        finally:
            self.videos_widget.clear_selected_videos()

    def _yt_hook(self, d):
        """ Callback function for yt dl """

        if d['status'] == 'downloading':
            self.download = True
            percent = d['downloaded_bytes'] / d['total_bytes']
            self.info_bar.set_info_text("Downloading ... Total bytes:%s Speed:%s ETA:%s" %
                                        (d['_total_bytes_str'],
                                         d['_speed_str'], d['_eta_str']),
                                        percent)
        elif d['status'] == 'error':
            self.download = False
            self.info_bar.set_info_text("An error occured...")
        elif d['status'] == 'finished':
            self.download = False
            file = d['filename']
            self.info_bar.set_info_text("Done...%s" % file)

        elif d['status'] == 'exists':
            self.info_bar.set_info_text("Already in library.")

    def open_in_browser(self):
        """ Opens the video on youtube """

        if not self.videos_widget.is_focused:
            return
        video = self.videos_widget.get_current_index_item()
        if video and video.link:
            webbrowser.open_new_tab(video.link)

    def open_in_vlc(self):
        """ Opens the link in VLC media player """

        video = self.videos_widget.get_current_index_item()
        if video and video.link:
            try:
                subprocess.Popen(['vlc', str(video.link)],
                                 stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            except Exception:
                self.info_bar.set_info_text("Can't open in vlc, probably not installed or inaccessible...")

        if self.yet_config.get_section("common").getboolean("auto_markasread"):
            self.mark_as_read()

    def mark_as_read(self, selected_videos=False):
        """ Sets the video as mark as read """
        if not selected_videos:
            video = self.videos_widget.get_current_index_item()
            if video.read:
                video.read = False
                self.videos_cache.delete(video.link)
            else:
                video.read = True
                self.videos_cache.add_to_cache(video.link)
            self.videos_widget.display()
        else:
            for video in self.videos_widget.selected_videos:
                video.read = True
                self.videos_cache.add_to_cache(video.link)
            self.videos_widget.display()





class Rect():
    def __init__(self, *args, **kwargs):
        rect = kwargs.get('rect')
        if rect:
            self.w = rect.w
            self.h = rect.h
            self.x = rect.x
            self.y = rect.y
        else:
            self.w = args[0]
            self.h = args[1]
            self.x = args[2]
            self.y = args[3]



from .widget import Widget
import curses
import textwrap


class InfoBar(Widget):
    ''' Info bar widget '''

    def __init__(self, window, content, header, rect, config):
        super().__init__(window, content, header, rect, config)
        self.info_text = ""
        self.progress = 1.0
        self.color = curses.color_pair(config.getint('fg', fallback=4)) | curses.A_REVERSE
        self.colorprogress = curses.color_pair(config.getint('progressbg', fallback=0)) | curses.A_REVERSE

    def set_info_text(self, text, progress=1.0):
        self.info_text = text
        self.progress = progress
        self.display()

    def display(self):
        self.subwindow.erase()
        text = textwrap.shorten(self.info_text, self.rect.w - 2)
        text += ' ' * (self.rect.w - len(text) - 2)
        try:
            length = len(text)
            done = int((length * self.progress))
            self.subwindow.addstr(0, 1, text[0:done], self.color)
            self.subwindow.addstr(0, 1 + done, text[done:length], self.colorprogress)
        except curses.error:
            # TODO log
            pass

        self.subwindow.refresh()



import setuptools
import yet

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ye-tui",
    version=yet.__version__,
    author=yet.__author__,
    author_email=yet.__email__,
    license=yet.__license__,
    description="yet is a console (kind of rss) application to download youtube videos.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nevarman/yet",
    packages=setuptools.find_packages(),
    package_data={
        'yet': [
            'config/yet.conf',
        ],
    },
    classifiers=[
        'Environment :: Console',
        'Environment :: Console :: Curses',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3.6',
        'Topic :: Desktop Environment',
        'Topic :: Utilities',
    ],
    entry_points='''
        [console_scripts]
        yet=yet.main:main
    ''',
    install_requires=[
        'xmltodict',
        'youtube-dl'
    ],
    python_requires='>=3.6',
)



from __future__ import unicode_literals
import youtube_dl
from yet.config import config
import threading
from pathlib import Path


class MyLogger(object):
    # TODO add file logger
    def debug(self, msg):
        pass

    def warning(self, msg):
        pass

    def error(self, msg):
        pass


class YtdlWrapper():

    def __init__(self, hook, ytconfig):
        self.hook = hook
        self.config = ytconfig
        self.output_path = self.config.get('output_path', fallback=config.get_videos_dir())
        self.extension = self.config.get('video_format', fallback='mp4')
        self.ydl_opts = {
            'format': 'bestvideo[ext={ex}]+bestaudio[ext=m4a]/best[ext={ex}]/best'.format(ex=self.extension),
            'outtmpl': self.output_path + '%(title)s.%(ext)s',
            # 'postprocessors': [{
            #     'key': 'FFmpegExtractAudio',
            #     'preferredcodec': 'mp3',
            #     'preferredquality': '192',
            # }],
            'logger': MyLogger(),
            'progress_hooks': [self.hook]}

    def get(self, url_list):
        thread = threading.Thread(target=self.start_t, args=(url_list,))
        thread.setDaemon(True)
        thread.start()

    def start_t(self, url_list: list):
        with youtube_dl.YoutubeDL(self.ydl_opts) as ydl:
            for u in url_list:
                info = ydl.extract_info(u, download=False)
                name = self.output_path + u'{title}.{ext}'.format(title=info['title'], ext=self.extension)
                if Path(name).is_file():
                    self.hook({'status': 'exists'})
                    url_list.remove(u)
            try:
                ydl.download(url_list)
            except Exception:
                self.hook({'status': 'error'})
                # TODO add logger




import urllib.request as urllib2
import xmltodict


class Entry:

    def __init__(self, entry):
        self.title = entry['title']
        self.link = entry['link']['@href']
        self.published = entry['published'].replace("T", " / ") if entry['published'] is not None else "-"
        self.updated = entry['updated'].replace("T", " / ") if entry['updated'] is not None else "-"
        self.thumbnail = entry['media:group']['media:thumbnail']['@url']
        self.description = entry['media:group']['media:description'].replace(
                '\n', '') if entry['media:group']['media:description'] is not None else "No information..."
        self.views = entry['media:group']['media:community']['media:statistics']['@views']
        self.read = False

    def __str__(self):
        return self.title

    def get_extended_info(self) -> list:
        return [self.title, self.description, f"Published:{self.published}",
                f"Updated:{self.updated}", f"Views:{self.views}"]


class Feed:

    def __init__(self, data: dict):
        self.data = data
        self.entries = []
        feed = self.data.get('feed')
        if feed is None:
            return
        # infos
        self.author = feed['author']['name']
        self.url = feed['author']['uri']
        # entries
        e = feed.get('entry')
        if e is None:
            return
        for i in e:
            entry = Entry(i)
            # TODO check cache read
            self.entries.append(entry)


class Channel:

    def __init__(self, c_id):
        self.channel_id = c_id
        self.feed = self.load_feed()
        self.url = self.feed.url
        self.author = self.feed.author
        self.entries = self.feed.entries

    def load_feed(self):
        uri = f'https://www.youtube.com/feeds/videos.xml?channel_id={self.channel_id}'
        with urllib2.urlopen(uri) as response:
            data = response.read()
            data = xmltodict.parse(data)
            return Feed(data)

    def __str__(self):
        return self.author

    def __len__(self):
        return len(self.author)


class Subscriptions:

    def __init__(self, channel_ids: list, callback=None):
        self.channel_ids = channel_ids
        self.subscriptions = []
        self.callback = callback
        for i in range(len(channel_ids)):
            channel = Channel(channel_ids[i])
            if len(channel.entries) > 0:
                self.subscriptions.append(channel)
            # send callback
            if self.callback is not None:
                self.callback(i + 1, len(channel_ids), self.get_sorted_subscriptions())

    def get_sorted_subscriptions(self):
        return sorted(
            self.subscriptions, key=lambda ch: str(ch))

    def get_channel(self, c_id) -> Channel:
        if c_id > len(self.subscriptions) - 1 or c_id < 0:
            raise IndexError('id should be in range of subs list')
        return self.subscriptions[c_id]



from .scrollablewidget import ScrollableWidget
import textwrap
import curses


class VideosWidget(ScrollableWidget):

    def __init__(self, window, content, header, rect, box=True):
        super().__init__(window, content, header, rect, box)
        self.selected_videos = []

    def select_video(self):
        if not self.is_focused:
            return
        video = self.get_current_index_item()
        if video in self.selected_videos:
            self.selected_videos.remove(video)
        else:
            self.selected_videos.append(video)
        self.display()

    def clear_selected_videos(self):
        self.selected_videos = []
        self.display

    def get_urls_to_download(self):
        if len(self.selected_videos) > 0:
            return [str(feed.link) for feed in self.selected_videos]
        elif self.is_focused:
            return [str(self.get_current_index_item().link)]

        return None

    def _draw_items(self):
        # header
        start_y = self.draw_header()

        # draw items
        for index, item in enumerate(self.content[self.top:self.top + self.max_lines]):
            # Highlight the current cursor line
            color = self.selected_color if index == self.current else self.color
            item_text = '* ' + \
                str(item) if item in self.selected_videos else str(item)
            if item in self.selected_videos:
                color = color | curses.A_BOLD
            if item.read:
                color = color | curses.A_DIM
            short_text = textwrap.shorten(item_text, self.rect.w - self._TEXT_WRAP_PADDING)
            # until the end of line
            short_text += " " * (self.rect.w - 2 - len(short_text))
            self.subwindow.addstr(
                index + start_y,
                1,
                short_text,
                color)



from .scrollablewidget import ScrollableWidget, Focusable


class InfoWidget(ScrollableWidget, Focusable):

    def update_content(self, content, scroll_to_zero=True):
        new_content = []
        for i in content:
            new_content.extend(self._wrap(
                i, self.rect.w - self._TEXT_WRAP_PADDING))
            new_content.append('')
        self.content = new_content
        self.bottom = len(self.content)
        self.current = 0
        self.display()

    def _wrap(self, s, w):
        return [s[i:i + w].strip().lstrip() for i in range(0, len(s), w)]
